@@ -1,5 +1,6 @@
 /* Functions for dealing with '\0' separated arg vectors.
-   Copyright (C) 1995-1998, 2000-2002, 2006, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2002, 2006, 2008-2010 Free Software
+   Foundation, Inc.
    This file is part of the GNU C Library.
 
    This program is free software; you can redistribute it and/or modify
@@ -219,7 +220,7 @@ argz_count (const char *argz, size_t len)
   size_t count = 0;
   while (len > 0)
     {
-      size_t part_len = strlen(argz);
+      size_t part_len = strlen (argz);
       argz += part_len + 1;
       len -= part_len + 1;
       count++;
