@@ -41,6 +41,7 @@ static const char *const s_load   = "load";
 static const char *const s_pathx  = "pathx";
 static const char *const s_error  = "error";
 static const char *const s_pos    = "pos";
+static const char *const s_vars   = "variables";
 
 #define TREE_HIDDEN(tree) ((tree)->label == NULL)
 
@@ -62,7 +63,8 @@ static const char *const errcodes[] = {
     "Too many matches for path expression",             /* AUG_EMMATCH */
     "Syntax error in lens definition",                  /* AUG_ESYNTAX */
     "Lens not found",                                   /* AUG_ENOLENS */
-    "Multiple transforms"                               /* AUG_EMXFM */
+    "Multiple transforms",                              /* AUG_EMXFM */
+    "Node has no span info"                             /* AUG_ENOSPAN */
 };
 
 static void tree_mark_dirty(struct tree *tree) {
@@ -73,8 +75,7 @@ static void tree_mark_dirty(struct tree *tree) {
     tree->dirty = 1;
 }
 
-/* Clear the dirty flag in the whole TREE */
-static void tree_clean(struct tree *tree) {
+void tree_clean(struct tree *tree) {
     if (tree->dirty) {
         list_for_each(c, tree->children)
             tree_clean(c);
@@ -126,7 +127,7 @@ struct tree *tree_find(struct augeas *aug, const char *path) {
     struct tree *result = NULL;
     int r;
 
-    pathx_parse(aug->origin, aug->error, path, true, aug->symtab, &p);
+    p = pathx_aug_parse(aug, aug->origin, path, true);
     ERR_BAIL(aug);
 
     r = pathx_find_one(p, &result);
@@ -146,27 +147,45 @@ struct tree *tree_find_cr(struct augeas *aug, const char *path) {
     struct tree *result = NULL;
     int r;
 
-    pathx_parse(aug->origin, aug->error, path, true, aug->symtab, &p);
+    p = pathx_aug_parse(aug, aug->origin, path, true);
     ERR_BAIL(aug);
 
     r = pathx_expand_tree(p, &result);
     ERR_BAIL(aug);
+    ERR_THROW(r < 0, aug, AUG_EINTERNAL, "pathx_expand_tree failed");
  error:
     free_pathx(p);
     return result;
 }
 
-int tree_set_value(struct tree *tree, const char *value) {
+void tree_store_value(struct tree *tree, char **value) {
+    if (streqv(tree->value, *value)) {
+        free(*value);
+        *value = NULL;
+        return;
+    }
     if (tree->value != NULL) {
         free(tree->value);
         tree->value = NULL;
     }
-    if (value != NULL) {
-        tree->value = strdup(value);
-        if (tree->value == NULL)
-            return -1;
+    if (*value != NULL) {
+        tree->value = *value;
+        *value = NULL;
     }
     tree_mark_dirty(tree);
+}
+
+int tree_set_value(struct tree *tree, const char *value) {
+    char *v = NULL;
+
+    if (streqv(tree->value, value))
+        return 0;
+    if (value != NULL) {
+        v = strdup(value);
+        if (v == NULL)
+            return -1;
+    }
+    tree_store_value(tree, &v);
     return 0;
 }
 
@@ -187,14 +206,16 @@ static void store_pathx_error(const struct augeas *aug) {
     tree_set_value(tpos, aug->error->details);
 }
 
-static struct pathx *parse_user_pathx(const struct augeas *aug,
-                                      bool need_nodeset,
-                                      const char *path) {
+struct pathx *pathx_aug_parse(const struct augeas *aug,
+                              struct tree *tree,
+                              const char *path, bool need_nodeset) {
     struct pathx *result;
     struct error *err = err_of_aug(aug);
 
-    pathx_parse(aug->origin, err, path, need_nodeset, aug->symtab,
-                &result);
+    if (tree == NULL)
+        tree = aug->origin;
+
+    pathx_parse(tree, err, path, need_nodeset, aug->symtab, &result);
     return result;
 }
 
@@ -305,10 +326,7 @@ static void api_entry(const struct augeas *aug) {
     if (aug->api_entries > 1)
         return;
 
-    err->code = AUG_NOERROR;
-    err->minor = 0;
-    FREE(err->details);
-    err->minor_details = NULL;
+    reset_error(err);
     save_locale((struct augeas *) aug);
 }
 
@@ -405,8 +423,16 @@ struct augeas *aug_init(const char *root, const char *loadpath,
     } else {
         aug_set(result, AUGEAS_META_SAVE_MODE, AUG_SAVE_OVERWRITE_TEXT);
     }
-    /* Make sure we always have /files */
-    aug_set(result, AUGEAS_FILES_TREE, NULL);
+
+    if (flags & AUG_ENABLE_SPAN) {
+        aug_set(result, AUGEAS_SPAN_OPTION, AUG_ENABLE);
+    } else {
+        aug_set(result, AUGEAS_SPAN_OPTION, AUG_DISABLE);
+    }
+
+    /* Make sure we always have /files and /augeas/variables */
+    tree_path_cr(result->origin, 1, s_files);
+    tree_path_cr(result->origin, 2, s_augeas, s_vars);
 
     if (interpreter_init(result) == -1)
         goto error;
@@ -440,25 +466,110 @@ void tree_unlink_children(struct augeas *aug, struct tree *tree) {
         tree_unlink(tree->children);
 }
 
+static void tree_mark_files(struct tree *tree) {
+    if (tree_child(tree, "path") != NULL) {
+        tree_mark_dirty(tree);
+    } else {
+        list_for_each(c, tree->children) {
+            tree_mark_files(c);
+        }
+    }
+}
+
+static void tree_rm_dirty_files(struct augeas *aug, struct tree *tree) {
+    struct tree *p;
+
+    if (!tree->dirty)
+        return;
+
+    if ((p = tree_child(tree, "path")) != NULL) {
+        aug_rm(aug, p->value);
+        tree_unlink(tree);
+    } else {
+        struct tree *c = tree->children;
+        while (c != NULL) {
+            struct tree *next = c->next;
+            tree_rm_dirty_files(aug, c);
+            c = next;
+        }
+    }
+}
+
+static void tree_rm_dirty_leaves(struct augeas *aug, struct tree *tree,
+                                 struct tree *protect) {
+    if (! tree->dirty)
+        return;
+
+    struct tree *c = tree->children;
+    while (c != NULL) {
+        struct tree *next = c->next;
+        tree_rm_dirty_leaves(aug, c, protect);
+        c = next;
+    }
+
+    if (tree != protect && tree->children == NULL)
+        tree_unlink(tree);
+}
+
 int aug_load(struct augeas *aug) {
+    const char *option = NULL;
     struct tree *meta = tree_child_cr(aug->origin, s_augeas);
     struct tree *meta_files = tree_child_cr(meta, s_files);
     struct tree *files = tree_child_cr(aug->origin, s_files);
     struct tree *load = tree_child_cr(meta, s_load);
+    struct tree *vars = tree_child_cr(meta, s_vars);
 
     api_entry(aug);
 
     ERR_NOMEM(load == NULL, aug);
 
-    tree_unlink_children(aug, meta_files);
-    tree_unlink_children(aug, files);
+    /* To avoid unnecessary loads of files, we reload an existing file in
+     * several steps:
+     * (1) mark all file nodes under /augeas/files as dirty (and only those)
+     * (2) process all files matched by a lens; we check (in
+     *     transform_load) if the file has been modified. If it has, we
+     *     reparse it. Either way, we clear the dirty flag. We also need to
+     *     reread the file if part or all of it has been modified in the
+     *     tree but not been saved yet
+     * (3) remove all files from the tree that still have a dirty entry
+     *     under /augeas/files. Those files are not processed by any lens
+     *     anymore
+     * (4) Remove entries from /augeas/files and /files that correspond
+     *     to directories without any files of interest
+     */
+
+    /* update flags according to option value */
+    if (aug_get(aug, AUGEAS_SPAN_OPTION, &option) == 1) {
+        if (strcmp(option, AUG_ENABLE) == 0) {
+            aug->flags |= AUG_ENABLE_SPAN;
+        } else {
+            aug->flags &= ~AUG_ENABLE_SPAN;
+        }
+    }
+
+    tree_clean(meta_files);
+    tree_mark_files(meta_files);
 
     list_for_each(xfm, load->children) {
         if (transform_validate(aug, xfm) == 0)
             transform_load(aug, xfm);
     }
+
+    /* This makes it possible to spot 'directories' that are now empty
+     * because we removed their file contents */
+    tree_clean(files);
+
+    tree_rm_dirty_files(aug, meta_files);
+    tree_rm_dirty_leaves(aug, meta_files, meta_files);
+    tree_rm_dirty_leaves(aug, files, files);
+
     tree_clean(aug->origin);
 
+    list_for_each(v, vars->children) {
+        aug_defvar(aug, v->label, v->value);
+        ERR_BAIL(aug);
+    }
+
     api_exit(aug);
     return 0;
  error:
@@ -490,7 +601,7 @@ int aug_get(const struct augeas *aug, const char *path, const char **value) {
 
     api_entry(aug);
 
-    p = parse_user_pathx((struct augeas *) aug, true, path);
+    p = pathx_aug_parse(aug, aug->origin, path, true);
     ERR_BAIL(aug);
 
     if (value != NULL)
@@ -513,23 +624,10 @@ int aug_get(const struct augeas *aug, const char *path, const char **value) {
     return -1;
 }
 
-int aug_defvar(augeas *aug, const char *name, const char *expr) {
-    struct pathx *p = NULL;
-    int result = -1;
-
-    api_entry(aug);
-
-    if (expr == NULL) {
-        result = pathx_symtab_undefine(&(aug->symtab), name);
-    } else {
-        p = parse_user_pathx((struct augeas *) aug, false, expr);
-        ERR_BAIL(aug);
-        result = pathx_symtab_define(&(aug->symtab), name, p);
-    }
-    ERR_BAIL(aug);
-
+static void record_var_meta(struct augeas *aug, const char *name,
+                            const char *expr) {
     /* Record the definition of the variable */
-    struct tree *tree = tree_path_cr(aug->origin, 2, "augeas", "variables");
+    struct tree *tree = tree_path_cr(aug->origin, 2, s_augeas, s_vars);
     ERR_NOMEM(tree == NULL, aug);
     if (expr == NULL) {
         tree = tree_child(tree, name);
@@ -540,6 +638,27 @@ int aug_defvar(augeas *aug, const char *name, const char *expr) {
         ERR_NOMEM(tree == NULL, aug);
         tree_set_value(tree, expr);
     }
+ error:
+    return;
+}
+
+int aug_defvar(augeas *aug, const char *name, const char *expr) {
+    struct pathx *p = NULL;
+    int result = -1;
+
+    api_entry(aug);
+
+    if (expr == NULL) {
+        result = pathx_symtab_undefine(&(aug->symtab), name);
+    } else {
+        p = pathx_aug_parse(aug, aug->origin, expr, false);
+        ERR_BAIL(aug);
+        result = pathx_symtab_define(&(aug->symtab), name, p);
+    }
+    ERR_BAIL(aug);
+
+    record_var_meta(aug, name, expr);
+    ERR_BAIL(aug);
  error:
     free_pathx(p);
     api_exit(aug);
@@ -560,21 +679,32 @@ int aug_defnode(augeas *aug, const char *name, const char *expr,
     if (created == NULL)
         created = &cr;
 
-    p = parse_user_pathx((struct augeas *) aug, false, expr);
+    p = pathx_aug_parse(aug, aug->origin, expr, false);
     ERR_BAIL(aug);
 
-    r = pathx_expand_tree(p, &tree);
-    if (r < 0)
-        goto done;
+    if (pathx_first(p) == NULL) {
+        r = pathx_expand_tree(p, &tree);
+        if (r < 0)
+            goto done;
+        *created = 1;
+    } else {
+        *created = 0;
+    }
 
-    *created = r > 0;
     if (*created) {
         r = tree_set_value(tree, value);
         if (r < 0)
             goto done;
         result = pathx_symtab_assign_tree(&(aug->symtab), name, tree);
+        char *e = path_of_tree(tree);
+        ERR_NOMEM(e == NULL, aug)
+        record_var_meta(aug, name, e);
+        free(e);
+        ERR_BAIL(aug);
     } else {
         result = pathx_symtab_define(&(aug->symtab), name, p);
+        record_var_meta(aug, name, expr);
+        ERR_BAIL(aug);
     }
 
  done:
@@ -606,7 +736,7 @@ int aug_set(struct augeas *aug, const char *path, const char *value) {
 
     api_entry(aug);
 
-    p = parse_user_pathx(aug, true, path);
+    p = pathx_aug_parse(aug, aug->origin, path, true);
     ERR_BAIL(aug);
 
     result = tree_set(p, value) == NULL ? -1 : 0;
@@ -622,13 +752,12 @@ int aug_set(struct augeas *aug, const char *path, const char *value) {
 int aug_setm(struct augeas *aug, const char *base,
              const char *sub, const char *value) {
     struct pathx *bx = NULL, *sx = NULL;
-    struct error *err = err_of_aug(aug);
     struct tree *bt, *st;
     int result, r;
 
     api_entry(aug);
 
-    bx = parse_user_pathx(aug, true, base);
+    bx = pathx_aug_parse(aug, aug->origin, base, true);
     ERR_BAIL(aug);
 
     if (sub != NULL && STREQ(sub, "."))
@@ -638,7 +767,7 @@ int aug_setm(struct augeas *aug, const char *base,
     for (bt = pathx_first(bx); bt != NULL; bt = pathx_next(bx)) {
         if (sub != NULL) {
             /* Handle subnodes of BT */
-            pathx_parse(bt, err, sub, true, aug->symtab, &sx);
+            sx = pathx_aug_parse(aug, bt, sub, true);
             ERR_BAIL(aug);
             if (pathx_first(sx) != NULL) {
                 /* Change existing subnodes matching SUB */
@@ -706,7 +835,7 @@ int aug_insert(struct augeas *aug, const char *path, const char *label,
 
     api_entry(aug);
 
-    p = parse_user_pathx(aug, true, path);
+    p = pathx_aug_parse(aug, aug->origin, path, true);
     ERR_BAIL(aug);
 
     result = tree_insert(p, label, before);
@@ -751,6 +880,8 @@ static void free_tree_node(struct tree *tree) {
     if (tree == NULL)
         return;
 
+    if (tree->span != NULL)
+        free_span(tree->span);
     free(tree->label);
     free(tree->value);
     free(tree);
@@ -820,7 +951,7 @@ int aug_rm(struct augeas *aug, const char *path) {
 
     api_entry(aug);
 
-    p = parse_user_pathx(aug, true, path);
+    p = pathx_aug_parse(aug, aug->origin, path, true);
     ERR_BAIL(aug);
 
     result = tree_rm(p);
@@ -834,13 +965,70 @@ int aug_rm(struct augeas *aug, const char *path) {
     return -1;
 }
 
-int tree_replace(struct tree *origin, const char *path, struct tree *sub) {
+int aug_span(struct augeas *aug, const char *path, char **filename,
+        uint *label_start, uint *label_end, uint *value_start, uint *value_end,
+        uint *span_start, uint *span_end) {
+    struct pathx *p = NULL;
+    int result = -1;
+    struct tree *tree = NULL;
+    struct span *span;
+
+    api_entry(aug);
+
+    p = pathx_aug_parse(aug, aug->origin, path, true);
+    ERR_BAIL(aug);
+
+    tree = pathx_first(p);
+    ERR_BAIL(aug);
+
+    ERR_THROW(tree == NULL, aug, AUG_ENOMATCH, "No node matching %s", path);
+    ERR_THROW(tree->span == NULL, aug, AUG_ENOSPAN, "No span info for %s", path);
+    ERR_THROW(pathx_next(p) != NULL, aug, AUG_EMMATCH, "Multiple nodes match %s", path);
+
+    span = tree->span;
+
+    if (label_start != NULL)
+        *label_start = span->label_start;
+
+    if (label_end != NULL)
+        *label_end = span->label_end;
+
+    if (value_start != NULL)
+        *value_start = span->value_start;
+
+    if (value_end != NULL)
+        *value_end = span->value_end;
+
+    if (span_start != NULL)
+        *span_start = span->span_start;
+
+    if (span_end != NULL)
+        *span_end = span->span_end;
+
+    /* We are safer here, make sure we have a filename */
+    if (filename != NULL) {
+        if (span->filename == NULL || span->filename->str == NULL) {
+            *filename = strdup("");
+        } else {
+            *filename = strdup(span->filename->str);
+        }
+        ERR_NOMEM(*filename == NULL, aug);
+    }
+
+    result = 0;
+ error:
+    free_pathx(p);
+    api_exit(aug);
+    return result;
+}
+
+int tree_replace(struct augeas *aug, const char *path, struct tree *sub) {
     struct tree *parent;
     struct pathx *p = NULL;
     int r;
 
-    if (pathx_parse(origin, NULL, path, true, NULL, &p) != PATHX_NOERROR)
-        goto error;
+    p = pathx_aug_parse(aug, aug->origin, path, true);
+    ERR_BAIL(aug);
 
     r = tree_rm(p);
     if (r == -1)
@@ -869,10 +1057,10 @@ int aug_mv(struct augeas *aug, const char *src, const char *dst) {
     api_entry(aug);
 
     ret = -1;
-    s = parse_user_pathx(aug, true, src);
+    s = pathx_aug_parse(aug, aug->origin, src, true);
     ERR_BAIL(aug);
 
-    d = parse_user_pathx(aug, true, dst);
+    d = pathx_aug_parse(aug, aug->origin, dst, true);
     ERR_BAIL(aug);
 
     r = find_one_node(s, &ts);
@@ -928,7 +1116,7 @@ int aug_match(const struct augeas *aug, const char *pathin, char ***matches) {
         pathin = "/*";
     }
 
-    p = parse_user_pathx((struct augeas *) aug, true, pathin);
+    p = pathx_aug_parse(aug, aug->origin, pathin, true);
     ERR_BAIL(aug);
 
     for (tree = pathx_first(p); tree != NULL; tree = pathx_next(p)) {
@@ -1028,7 +1216,6 @@ static int tree_save(struct augeas *aug, struct tree *tree,
 /* Reset the flags based on what is set in the tree. */
 static int update_save_flags(struct augeas *aug) {
     const char *savemode ;
-    int noop = 0 ;
 
     aug_get(aug, AUGEAS_META_SAVE_MODE, &savemode);
     if (savemode == NULL)
@@ -1041,7 +1228,6 @@ static int update_save_flags(struct augeas *aug) {
         aug->flags |= AUG_SAVE_BACKUP;
     } else if (STREQ(savemode, AUG_SAVE_NOOP_TEXT)) {
         aug->flags |= AUG_SAVE_NOOP ;
-        noop = 1 ;
     } else if (STRNEQ(savemode, AUG_SAVE_OVERWRITE_TEXT)) {
         return -1;
     }
@@ -1069,8 +1255,8 @@ static int unlink_removed_files(struct augeas *aug,
         if (tf == NULL) {
             /* Unlink all files in tm */
             struct pathx *px = NULL;
-            if (pathx_parse(tm, err_of_aug(aug), file_nodes, true, NULL, &px)
-                != PATHX_NOERROR) {
+            if (pathx_parse(tm, err_of_aug(aug), file_nodes, true,
+                            aug->symtab, &px) != PATHX_NOERROR) {
                 result = -1;
                 continue;
             }
@@ -1228,7 +1414,7 @@ int aug_print(const struct augeas *aug, FILE *out, const char *pathin) {
         pathin = "/*";
     }
 
-    p = parse_user_pathx((struct augeas *) aug, true, pathin);
+    p = pathx_aug_parse(aug, aug->origin, pathin, true);
     ERR_BAIL(aug);
 
     result = print_tree(out, p, 0);
