@@ -41,8 +41,11 @@ static const char *const progname = "augtool";
 static unsigned int flags = AUG_NONE;
 const char *root = NULL;
 char *loadpath = NULL;
-int echo = 0;
+const char *inputfile = NULL;
+int echo = 0;         /* Gets also changed in main_loop */
 bool print_version = false;
+bool auto_save = false;
+bool interactive = false;
 
 /*
  * General utilities
@@ -57,6 +60,8 @@ static char *cleanstr(char *path, const char sep) {
 }
 
 static char *cleanpath(char *path) {
+    if (STREQ(path, "/"))
+        return path;
     return cleanstr(path, SEP);
 }
 
@@ -78,6 +83,8 @@ struct command_opt_def {
 
 #define CMD_OPT_DEF_LAST { .type = CMD_NONE, .name = NULL }
 
+#define AUGTOOL_PROMPT "augtool> "
+
 /* Handlers return one of these */
 enum command_result {
     CMD_RES_OK,
@@ -510,10 +517,12 @@ static const struct command_def cmd_match_def = {
 static void cmd_rm(struct command *cmd) {
     int cnt;
     const char *path = arg_value(cmd, "path");
-    printf("rm : %s", path);
+    if (echo)
+        printf("rm : %s", path);
     cnt = aug_rm(aug, path);
     err_check(cmd);
-    printf(" %d\n", cnt);
+    if (echo)
+        printf(" %d\n", cnt);
 }
 
 static const struct command_opt_def cmd_rm_opts[] = {
@@ -623,6 +632,84 @@ static const struct command_def cmd_setm_def = {
     "BASE will be modified."
 };
 
+static void cmd_clearm(struct command *cmd) {
+    const char *base = arg_value(cmd, "base");
+    const char *sub  = arg_value(cmd, "sub");
+    int r;
+
+    r = aug_setm(aug, base, sub, NULL);
+    err_check(cmd);
+    if (r == -1)
+        printf ("Failed\n");
+}
+
+static const struct command_opt_def cmd_clearm_opts[] = {
+    { .type = CMD_PATH, .name = "base", .optional = false,
+      .help = "the base node" },
+    { .type = CMD_PATH, .name = "sub", .optional = false,
+      .help = "the subtree relative to the base" },
+    CMD_OPT_DEF_LAST
+};
+
+static const struct command_def cmd_clearm_def = {
+    .name = "clearm",
+    .opts = cmd_clearm_opts,
+    .handler = cmd_clearm,
+    .synopsis = "clear the value of multiple nodes",
+    .help = "Clear multiple nodes values in one operation. Find or create a"
+    " node matching SUB\n by interpreting SUB as a path expression relative"
+    " to each node matching\n BASE. If SUB is '.', the nodes matching "
+    "BASE will be modified."
+};
+
+static void cmd_span(struct command *cmd) {
+    const char *path = arg_value(cmd, "path");
+    int r;
+    uint label_start, label_end, value_start, value_end, span_start, span_end;
+    char *filename;
+    const char *option;
+    // FIXME: add check to see if AUG_ENABLE_SPAN is set
+
+    if (aug_get(aug, AUGEAS_SPAN_OPTION, &option) != 1) {
+        printf("Error: option " AUGEAS_SPAN_OPTION " not found\n");
+        return;
+    }
+    if (strcmp(AUG_DISABLE, option) == 0) {
+        printf("Span is not enabled. To enable, run commands:\n");
+        printf("set %s %s\n", AUGEAS_SPAN_OPTION, AUG_ENABLE);
+        printf("rm %s\n", AUGEAS_FILES_TREE);
+        printf("load\n");
+        return;
+    } else if (strcmp(AUG_ENABLE, option) != 0) {
+        printf("Error: option %s must be %s or %s\n", AUGEAS_SPAN_OPTION,
+               AUG_ENABLE, AUG_DISABLE);
+        return;
+    }
+    r = aug_span(aug, path, &filename, &label_start, &label_end, &value_start,
+                 &value_end, &span_start, &span_end);
+    err_check(cmd);
+    if (r == -1){
+        printf ("Failed\n");
+        return;
+    }
+    printf("%s label=(%i:%i) value=(%i:%i) span=(%i,%i)\n", filename,
+           label_start, label_end, value_start, value_end, span_start, span_end);
+}
+
+static const struct command_opt_def cmd_span_opts[] = {
+    { .type = CMD_PATH, .name = "path", .optional = false,
+      .help = "node path" },
+    CMD_OPT_DEF_LAST
+};
+
+static const struct command_def cmd_span_def = {
+    .name = "span",
+    .opts = cmd_span_opts,
+    .handler = cmd_span,
+    .synopsis = "get the filename, label and value position in the text of this node",
+    .help = "get the filename, label and value position in the text of this node"
+};
+
 static void cmd_defvar(struct command *cmd) {
     const char *name = arg_value(cmd, "name");
     const char *path = arg_value(cmd, "expr");
@@ -779,7 +866,8 @@ static void cmd_save(struct command *cmd) {
     } else {
         r = aug_match(aug, "/augeas/events/saved", NULL);
         if (r > 0) {
-            printf("Saved %d file(s)\n", r);
+            if (echo)
+                printf("Saved %d file(s)\n", r);
         } else if (r < 0) {
             printf("Error during match: %d\n", r);
         }
@@ -931,6 +1019,8 @@ static const struct command_def const *commands[] = {
     &cmd_save_def,
     &cmd_set_def,
     &cmd_setm_def,
+    &cmd_clearm_def,
+    &cmd_span_def,
     &cmd_help_def,
     &cmd_def_last
 };
@@ -990,9 +1080,13 @@ static void usage(void) {
     fprintf(stderr, "  -r, --root ROOT    use ROOT as the root of the filesystem\n");
     fprintf(stderr, "  -I, --include DIR  search DIR for modules; can be given mutiple times\n");
     fprintf(stderr, "  -e, --echo         echo commands when reading from a file\n");
-    fprintf(stderr, "  --nostdinc         do not search the builtin default directories for modules\n");
-    fprintf(stderr, "  --noload           do not load any files into the tree on startup\n");
-    fprintf(stderr, "  --noautoload       do not autoload modules from the search path\n");
+    fprintf(stderr, "  -f, --file FILE    read commands from FILE\n");
+    fprintf(stderr, "  -s, --autosave     automatically save at the end of instructions\n");
+    fprintf(stderr, "  -i, --interactive  run an interactive shell after evaluating the commands in STDIN and FILE\n");
+    fprintf(stderr, "  -S, --nostdinc     do not search the builtin default directories for modules\n");
+    fprintf(stderr, "  -L, --noload       do not load any files into the tree on startup\n");
+    fprintf(stderr, "  -A, --noautoload   do not autoload modules from the search path\n");
+    fprintf(stderr, "  --span             load span positions for nodes related to a file\n");
     fprintf(stderr, "  --version          print version information and exit.\n");
 
     exit(EXIT_FAILURE);
@@ -1002,10 +1096,8 @@ static void parse_opts(int argc, char **argv) {
     int opt;
     size_t loadpathlen = 0;
     enum {
-        VAL_NO_STDINC = CHAR_MAX + 1,
-        VAL_NO_LOAD = VAL_NO_STDINC + 1,
-        VAL_NO_AUTOLOAD = VAL_NO_LOAD + 1,
-        VAL_VERSION = VAL_NO_AUTOLOAD + 1
+        VAL_VERSION = CHAR_MAX + 1,
+        VAL_SPAN = VAL_VERSION + 1
     };
     struct option options[] = {
         { "help",      0, 0, 'h' },
@@ -1015,15 +1107,19 @@ static void parse_opts(int argc, char **argv) {
         { "root",      1, 0, 'r' },
         { "include",   1, 0, 'I' },
         { "echo",      0, 0, 'e' },
-        { "nostdinc",  0, 0, VAL_NO_STDINC },
-        { "noload",    0, 0, VAL_NO_LOAD },
-        { "noautoload", 0, 0, VAL_NO_AUTOLOAD },
+        { "file",      1, 0, 'f' },
+        { "autosave",  0, 0, 's' },
+        { "interactive",  0, 0, 'i' },
+        { "nostdinc",  0, 0, 'S' },
+        { "noload",    0, 0, 'L' },
+        { "noautoload", 0, 0, 'A' },
+        { "span",      0, 0, VAL_SPAN },
         { "version",   0, 0, VAL_VERSION },
         { 0, 0, 0, 0}
     };
     int idx;
 
-    while ((opt = getopt_long(argc, argv, "hnbcr:I:e", options, &idx)) != -1) {
+    while ((opt = getopt_long(argc, argv, "hnbcr:I:ef:siSLA", options, &idx)) != -1) {
         switch(opt) {
         case 'c':
             flags |= AUG_TYPE_CHECK;
@@ -1046,19 +1142,31 @@ static void parse_opts(int argc, char **argv) {
         case 'e':
             echo = 1;
             break;
-        case VAL_NO_STDINC:
+        case 'f':
+            inputfile = optarg;
+            break;
+        case 's':
+            auto_save = true;
+            break;
+        case 'i':
+            interactive = true;
+            break;
+        case 'S':
             flags |= AUG_NO_STDINC;
             break;
-        case VAL_NO_LOAD:
+        case 'L':
             flags |= AUG_NO_LOAD;
             break;
-        case VAL_NO_AUTOLOAD:
+        case 'A':
             flags |= AUG_NO_MODL_AUTOLOAD;
             break;
         case VAL_VERSION:
             flags |= AUG_NO_MODL_AUTOLOAD;
             print_version = true;
             break;
+        case VAL_SPAN:
+            flags |= AUG_ENABLE_SPAN;
+            break;
         default:
             usage();
             break;
@@ -1112,23 +1220,77 @@ run_command(const char *line) {
 static int main_loop(void) {
     char *line = NULL;
     int ret = 0;
-    size_t len = 0;
+    char inputline [128];
     enum command_result code;
+    bool end_reached = false;
+    bool get_line = true;
+    bool in_interactive = false;
+    // make readline silent by default
+    rl_outstream = fopen("/dev/null", "w");
+
+    if (inputfile) {
+        if (freopen(inputfile, "r", stdin) == NULL) {
+            char *msg = NULL;
+            if (asprintf(&msg, "Failed to open %s", inputfile) < 0)
+                perror("Failed to open input file");
+            else
+                perror(msg);
+            return CMD_RES_ERR;
+        }
+    }
+
+    echo = echo || isatty(fileno(stdin));
+
+    if (echo)
+        rl_outstream = NULL;
 
     while(1) {
-        if (isatty(fileno(stdin))) {
-            line = readline("augtool> ");
+        if (get_line) {
+            line = readline(AUGTOOL_PROMPT);
         } else {
-            if (getline(&line, &len, stdin) == -1)
-                return ret;
-            if (echo)
-                printf("augtool> %s", line);
+            line = NULL;
         }
-        cleanstr(line, '\n');
+
         if (line == NULL) {
-            printf("\n");
+            if (!isatty(fileno(stdin)) && interactive && !in_interactive) {
+               in_interactive = true;
+               echo = true;
+               // reopen in and out streams
+               if ((rl_instream = fopen("/dev/tty", "r")) == NULL) {
+                   perror("Failed to open terminal for reading");
+                   return CMD_RES_ERR;
+               }
+               if (rl_outstream != NULL) {
+                   fclose(rl_outstream);
+                   rl_outstream = NULL;
+               }
+               if ((rl_outstream = fopen("/dev/stdout", "w")) == NULL) {
+                   perror("Failed to reopen stdout");
+                   return CMD_RES_ERR;
+               }
+               continue;
+            }
+
+            if (auto_save) {
+                strncpy(inputline, "save", sizeof(inputline));
+                line = inputline;
+                if (echo)
+                    printf("%s\n", line);
+                auto_save = false;
+            } else {
+                end_reached = true;
+            }
+            get_line = false;
+        }
+
+        cleanstr(line, '\n');
+
+        if (end_reached) {
+            if (echo)
+                printf("\n");
             return ret;
         }
+
         if (*line == '\0' || *line == '#')
             continue;
 
