@@ -575,6 +575,10 @@ static void parse_dot_link(FILE *fp, struct jmt_parse *parse,
             r = xasprintf(&lens_label, "<%d>", lnk->lens);
         else
             r = xasprintf(&lens_label, "%d", lnk->lens);
+        if (r < 0) {
+            fprintf(fp, "// Internal error generating lens_label\n");
+            return;
+        }
     }
     fprintf(fp, "    n%d_%d_%d [ label = \"(%d, %d)\"];\n",
             k, x->state->num, x->parent, x->state->num, x->parent);
@@ -754,6 +758,7 @@ build_nullable(struct jmt_parse *parse, ind_t pos,
                                    lens->children[i], lvl+1);
             break;
         case L_SUBTREE:
+        case L_SQUARE:
             build_nullable(parse, pos, visitor, lens->child, lvl+1);
             break;
         case L_STAR:
@@ -1014,7 +1019,7 @@ int jmt_visit(struct jmt_visitor *visitor, size_t *len) {
         struct item *x = set_item(parse, k, item);
         if (x->parent == 0 && returns(x->state, parse->jmt->lens)) {
             for (ind_t i = 0; i < x->nlinks; i++) {
-                if (is_complete(x->links + i)) {
+                if (is_complete(x->links + i) || is_scan(x->links + i)) {
                     if (debugging("cf.jmt.visit"))
                         printf("visit: found (%d, %d) in E_%d\n",
                                x->state->num, x->parent, k);
@@ -1230,6 +1235,11 @@ static void print_grammar(struct jmt *jmt, struct lens *lens) {
         printf("\n");
         print_grammar(jmt, lens->body);
         break;
+    case L_SQUARE:
+       print_lens_symbol(stdout, jmt, lens->child);
+       printf("\n");
+       print_grammar(jmt, lens->child);
+       break;
     default:
         BUG_ON(true, jmt, "Unexpected lens tag %d", lens->tag);
         break;
@@ -1271,6 +1281,7 @@ static void index_lenses(struct jmt *jmt, struct lens *lens) {
     case L_SUBTREE:
     case L_STAR:
     case L_MAYBE:
+    case L_SQUARE:
         index_lenses(jmt, lens->child);
         break;
     case L_REC:
@@ -1472,6 +1483,9 @@ static void conv_rhs(struct jmt *jmt, ind_t l) {
         conv(jmt, lens->child, &s, &e, &f);
         add_new_trans(jmt, s, e, EPS);
         break;
+    case L_SQUARE:
+       conv(jmt, lens->child, &s, &e, &f);
+       break;
     default:
         BUG_ON(true, jmt, "Unexpected lens tag %d", lens->tag);
     }
