@@ -226,7 +226,7 @@ static struct split *split_concat(struct state *state, struct lens *lens) {
         count = -1;
     if (count < 0) {
         regexp_match_error(state, lens, count, outer);
-        return NULL;
+        goto error;
     }
 
     struct tree *cur = outer->tree;
@@ -245,9 +245,14 @@ static struct split *split_concat(struct state *state, struct lens *lens) {
         reg += 1 + regexp_nsub(lens->children[i]->atype);
     }
     assert(reg < regs.num_regs);
+ done:
     free(regs.start);
     free(regs.end);
     return split;
+ error:
+    free_split(split);
+    split = NULL;
+    goto done;
 }
 
 static struct split *split_iter(struct state *state, struct lens *lens) {
@@ -414,6 +419,8 @@ static int skel_instance_of(struct lens *lens, struct skel *skel) {
         return 1;
     case L_REC:
         return skel_instance_of(lens->body, skel);
+    case L_SQUARE:
+        return skel->tag == L_SQUARE;
     default:
         BUG_ON(true, lens->info, "illegal lens tag %d", lens->tag);
         break;
@@ -461,7 +468,12 @@ static void put_del(ATTRIBUTE_UNUSED struct lens *lens, struct state *state) {
     assert(lens->tag == L_DEL);
     assert(state->skel != NULL);
     assert(state->skel->tag == L_DEL);
+    if (lens->string != NULL) {
     fprintf(state->out, "%s", state->skel->text);
+    } else {
+    /* L_DEL with NULL string: replicate the current key */
+        fprintf(state->out, "%s", state->key);
+    }
 }
 
 static void put_union(struct lens *lens, struct state *state) {
@@ -585,6 +597,13 @@ static void put_rec(struct lens *lens, struct state *state) {
     put_lens(lens->body, state);
 }
 
+static void put_square(struct lens *lens, struct state *state) {
+    struct skel *oldskel = state->skel;
+    state->skel = state->skel->skels;
+    put_lens(lens->child, state);
+    state->skel = oldskel;
+}
+
 static void put_lens(struct lens *lens, struct state *state) {
     if (state->error != NULL)
         return;
@@ -627,6 +646,9 @@ static void put_lens(struct lens *lens, struct state *state) {
     case L_REC:
         put_rec(lens, state);
         break;
+    case L_SQUARE:
+        put_square(lens, state);
+        break;
     default:
         assert(0);
         break;
@@ -639,8 +661,13 @@ static void create_subtree(struct lens *lens, struct state *state) {
 
 static void create_del(struct lens *lens, struct state *state) {
     assert(lens->tag == L_DEL);
+    if (lens->string != NULL) {
+        print_escaped_chars(state->out, lens->string->str);
+    } else {
+        /* L_DEL with NULL string: replicate the current key */
+        print_escaped_chars(state->out, state->key);
+    }
 
-    print_escaped_chars(state->out, lens->string->str);
 }
 
 static void create_union(struct lens *lens, struct state *state) {
@@ -749,6 +776,9 @@ static void create_lens(struct lens *lens, struct state *state) {
     case L_REC:
         create_rec(lens, state);
         break;
+    case L_SQUARE:
+        create_concat(lens->child, state);
+        break;
     default:
         assert(0);
         break;
