@@ -1729,6 +1729,7 @@ static int compile_test(struct term *term, struct ctx *ctx) {
         }
     }
  done:
+    reset_error(term->info->error);
     unref(actual, value);
     unref(expect, value);
     return ret;
@@ -1954,8 +1955,8 @@ int load_module_file(struct augeas *aug, const char *filename) {
         goto error;
 
     struct module *module = compile(term, aug);
-    if (module == NULL)
-        goto error;
+    ERR_THROW(module == NULL, aug, AUG_ESYNTAX,
+              "Failed to load %s", filename);
 
     list_append(aug->modules, module);
     result = 0;
