@@ -1,7 +1,7 @@
 /*
  * syntax.h: Data types to represent language syntax
  *
- * Copyright (C) 2007 - 2009 Red Hat Inc.
+ * Copyright (C) 2007-2010 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,10 +37,6 @@ void syntax_error(struct info *info, const char *format, ...)
 void fatal_error(struct info *info, const char *format, ...)
     ATTRIBUTE_FORMAT(printf, 2, 3);
 
-void assert_error_at(const char *srcfile, int srclineno, struct info *info,
-                     const char *format, ...)
-    ATTRIBUTE_FORMAT(printf, 4, 5);
-
 enum term_tag {
     A_MODULE,
     A_BIND,              /* Module scope binding of a name */
@@ -149,7 +145,8 @@ enum value_tag {
     V_TRANSFORM,
     V_NATIVE,
     V_EXN,
-    V_CLOS
+    V_CLOS,
+    V_UNIT
 };
 
 #define EXN(v) ((v)->tag == V_EXN)
@@ -158,6 +155,7 @@ struct value {
     unsigned int   ref;
     struct info   *info;
     enum value_tag tag;
+    /* Nothing in this union for V_UNIT */
     union {
         struct string  *string;  /* V_STRING */
         struct regexp  *regexp;  /* V_REGEXP */
@@ -189,7 +187,8 @@ enum type_tag {
     T_TREE,
     T_FILTER,
     T_TRANSFORM,
-    T_ARROW
+    T_ARROW,
+    T_UNIT
 };
 
 struct type {
@@ -228,6 +227,7 @@ struct term *make_term(enum term_tag tag, struct info *info);
 void free_term(struct term *term);
 struct term *make_param(char *name, struct type *type, struct info *info);
 struct value *make_value(enum value_tag tag, struct info *info);
+struct value *make_unit(struct info *info);
 struct term *make_app_term(struct term *func, struct term *arg,
                            struct info *info);
 struct term *make_app_ident(char *id, struct term *func, struct info *info);
